( cd usr/lib ; rm -rf libopendaap.so.0 )
( cd usr/lib ; ln -sf libopendaap.so.0.1.0 libopendaap.so.0 )
( cd usr/lib ; rm -rf libopendaap.so )
( cd usr/lib ; ln -sf libopendaap.so.0.1.0 libopendaap.so )
